# -*- mode: python; indent-tabs-mode: nil; tab-width: 3 -*-
# vim: set tabstop=3 shiftwidth=3 expandtab:
#
# Copyright (C) 2011 Tim Altmann, Christian Brandt
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#

from gamera.plugin import *

from gamera.toolkits.webcam.plugins import driverwrapper
import classwebcam
class WebcamCpp(classwebcam.Webcam):
   '''This class **Webcam** is a wrapper around the webcam C++-API and 
the main interface for people using the toolkit.'''

   def __init__(self, drivername):
      self.webcam = driverwrapper.CreateInstance(drivername)

   def __del__(self):
      driverwrapper.DeleteInstance(self.webcam)



   def initSession(self, width, height, device):
      '''Signature: 

   ``initSession (width, height, device)``
            
Starts a new camera session where *width* and *height* are the dimensions of the
resulting image and *device* is the identifier of the camera. If there is an 
exisiting camera session this session will be closed and a new session to the 
given camera is opened.'''
      self.width=width
      self.height=height
      return driverwrapper.InitWebcam(self.webcam, width, height, device)



   def snapshotFromWebcam(self):
      '''Signature:

   ``snapshotFromWebcam ()``

Returns a RGB Gamera image taken from the webcam.
'''
      image = driverwrapper.Snapshot(self.webcam)
      return image



   def closeSession(self):
      '''Signature:

   ``closeSession ()``

Closes a existing camera session and frees used resources.
'''
      driverwrapper.CloseWebcam(self.webcam)



   def getDevices(self):
      '''Signature:

   ``getDevices ()``

Returns a list of device names available on the current system.
'''
      
      devices = driverwrapper.GetDevices(self.webcam).split("[/]")
      if devices.count("No device") > 0:
         devices.remove("No device")
   
      return devices

Webcam = WebcamCpp
