/*
 *
 * Copyright (C) 2015 Bernd Böhm
 *
 * This toolkit is based on Tim Altmann's 2011 bachelor thesis
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
#include "webcamdshow.hpp"
#include <tchar.h>

#include <windows.h>
#include <dshow.h>

#include <stdio.h>
#include <iostream>
#include <string.h>
#include <strsafe.h>

//#include "gamera.hpp"

#pragma comment(lib,"user32.lib")
#pragma comment(lib,"Gdi32.lib")
#pragma comment (lib,"strmiids.lib")
#pragma comment (lib,"ole32.lib")
#pragma comment (lib,"oleaut32.lib")
namespace WebcamToolkit {


	Webcam* newWebcam() {
	   return new WebcamDShow;
	}
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	WebcamDShow::WebcamDShow()
	{		
		IBaseFilter* pSrc = NULL;
		bool DeviceSwitch = false;
		char* Device = NULL;		
		IVMRWindowlessControl * pWc = NULL;
		ICaptureGraphBuilder2 * g_pCapture = NULL;
	}
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	WebcamDShow::~WebcamDShow()
	{
			//closeSession(); already called by another function
	}
	

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	int WebcamDShow::initSession(int width, int height, char*device)
	{
		
		IMediaControl * g_pMC = NULL;
		IGraphBuilder * g_pGraph = NULL;
		IBaseFilter *pVmr = NULL;
		widthImage = width;
		heightImage = height;		
		Device = device;
		DeviceSwitch = true;		
		HRESULT hr;
	
	
		
		
		if(FAILED(CoInitializeEx(NULL, COINIT_APARTMENTTHREADED)))
    {
        printf(TEXT("CoInitialize Failed!\r\n"));   
        return 0;
    } 

		// Get DirectShow interfaces
		hr = CoCreateInstance (CLSID_FilterGraph, NULL, (CLSCTX_INPROC),
                           IID_IGraphBuilder, (void **) &g_pGraph);
		if (FAILED(hr))		
			return 0;

		// Create the capture graph builder
		hr = CoCreateInstance (CLSID_CaptureGraphBuilder2 , NULL, (CLSCTX_INPROC),
							   IID_ICaptureGraphBuilder2, (void **) &g_pCapture);
		if (FAILED(hr))
			return 0;
		
		// Obtain interfaces for media control and Video Window
		hr = g_pGraph->QueryInterface(IID_IMediaControl,(LPVOID *) &g_pMC);
		if (FAILED(hr))
			return 0;

	   
		
		//InitWindowlessVMR				
		// Create the VMR
		hr = CoCreateInstance(CLSID_VideoMixingRenderer, NULL, (CLSCTX_INPROC), IID_IBaseFilter, (void**)&pVmr);
		if (FAILED(hr))
		{
			return 0;
		}
		// Add the VMR to the filter graph.
		hr = g_pGraph->AddFilter(pVmr, L"Video Mixing Renderer");
		if (FAILED(hr))
		{
			pVmr->Release();
			return 0;
		}
		// set the rendering mode
		IVMRFilterConfig *pConfig;
		hr = pVmr->QueryInterface(IID_IVMRFilterConfig, (void**)&pConfig);
		if (SUCCEEDED(hr))
		{
			hr = pConfig->SetRenderingMode(VMRMode_Windowless);
			pConfig->Release();
		}
		if (SUCCEEDED(hr))
		{
			//Set the window
			hr = pVmr->QueryInterface(IID_IVMRWindowlessControl, (void**)&pWc);
			if (FAILED(hr))
			{
				return 0;
			}
		}
		pVmr->Release();
		
				
		
		// Attach the filter graph to the capture graph
		hr = g_pCapture->SetFiltergraph(g_pGraph);
		if (FAILED(hr))
		{
		   printf(TEXT("Failed to set capture filter graph!  hr=0x%x"), hr);
			return 0;
		}
		
		
		// Get Device and bind it to the filter, foo is only for calling the function
		std::string foo = getDevices();
		
		// Add Capture filter to our graph.
		hr = g_pGraph->AddFilter(pSrc, L"Video Capture");
		if (FAILED(hr))
		{
			printf(TEXT("Couldn't add the capture filter to the graph!  hr=0x%x\r\n\r\n") 
				TEXT("If you have a working video capture device, please make sure\r\n")
				TEXT("that it is connected and is not being used by another application.\r\n\r\n"), hr);				
			pSrc->Release();
			return 0;
		}
		
		//Add config filter to change the resolution
		IAMStreamConfig *g_pSC;
		hr = g_pCapture->FindInterface(&PIN_CATEGORY_CAPTURE, &MEDIATYPE_Video, pSrc, IID_IAMStreamConfig,(void**) &g_pSC);
		if (FAILED(hr))
		{
			hr = g_pCapture->FindInterface(&PIN_CATEGORY_CAPTURE,&MEDIATYPE_Interleaved, pSrc, IID_IAMStreamConfig, (void **)&g_pSC);
			printf("Second try\n");
			if (FAILED(hr))
			{
				printf("Couldn't bind the Streamconfigfilter to the capture graph\n");
			}
		}
		AM_MEDIA_TYPE *pmt;			
		hr = g_pSC->GetFormat(&pmt);		
		if (pmt->formattype == FORMAT_VideoInfo)
		{			
			// Check the buffer size.
			if (pmt->cbFormat >= sizeof(VIDEOINFOHEADER))
			{				
				VIDEOINFOHEADER *pVih = reinterpret_cast<VIDEOINFOHEADER*>(pmt->pbFormat);
				pVih->bmiHeader.biWidth=width;              
				pVih->bmiHeader.biHeight=height;
				hr = g_pSC->SetFormat(pmt);
				if (FAILED(hr))
				{
					printf("Resolution error\n");
					return 0;
				}
				/* Access VIDEOINFOHEADER members through pVih. */
			}
		}

		// Render the capture pin on the video capture filter		
		hr = g_pCapture->RenderStream (&PIN_CATEGORY_CAPTURE, &MEDIATYPE_Video,
									   pSrc, NULL, pVmr);
		if (FAILED(hr))
		{
		   printf(TEXT("Couldn't render the video capture stream.  hr=0x%x\r\n")
				  TEXT("The capture device may already be in use by another application.\r\n\r\n"), hr);				  
			pSrc->Release();
			return 0;
		}
		
		

		// Now that the filter has been added to the graph and we have
		// rendered its stream, we can release this reference to the filter.
		
		g_pSC->Release();
		
		
		
		
		
		g_pMC->Run();
		g_pMC->Release();        	
		g_pGraph->Release();
		
		
		
		CoUninitialize();
		return 1;
	}
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	unsigned char* WebcamDShow::snapshotFromWebcam()
	{
		BYTE *lpDib = NULL;
		unsigned char* picture = NULL;
		int i = 40;		//offset from lpDib from from infoheader to picture data, must be a multiple of 4	
		picture= new unsigned char[widthImage*heightImage*3];		
		HRESULT hr = pWc->GetCurrentImage(&lpDib);
				if (SUCCEEDED(hr))
				{
					picture = picture + ((heightImage) * (widthImage) * 3);					
					for (int r = 0; r < heightImage; ++r) {	
					picture = picture - (widthImage * 3);
						for (int c = 0; c < widthImage; ++c)
						{
						
							
							*picture++ = *(lpDib+i+2);
							*picture++ = *(lpDib+i+1);
							*picture++ = *(lpDib+i);							
								
							i = i+4;							
						}	
					picture = picture - (widthImage*3);		
					}
				}
						
		CoTaskMemFree(lpDib);
		return  picture;
	}
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	void WebcamDShow::closeSession()
	{
		DeviceSwitch = false;
		Device = NULL;
		pSrc->Release();
		pWc->Release();
		g_pCapture->Release();		
	}
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

	std::string ws2s(const std::wstring& s)
	{
		int len;
		int slength = (int)s.length() + 1;
		len = WideCharToMultiByte(CP_ACP, 0, s.c_str(), slength, 0, 0, 0, 0); 
		std::string r(len, '\0');
		WideCharToMultiByte(CP_ACP, 0, s.c_str(), slength, &r[0], len, 0, 0); 
		return r;
	}
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	std::string WebcamDShow::getDevices()
	{
		HRESULT hr;		
		IMoniker* pMoniker =NULL;
		ICreateDevEnum *pDevEnum =NULL;
		IEnumMoniker *pClassEnum = NULL;
		
				
	   
		// Create the system device enumerator
		hr = CoCreateInstance (CLSID_SystemDeviceEnum, NULL, (CLSCTX_INPROC),
							   IID_ICreateDevEnum, (void **) &pDevEnum);
		if (FAILED(hr))
		{
			printf(TEXT("Couldn't create system enumerator!  hr=0x%x"), hr);
			return "No device";
		}

		// Create an enumerator for the video capture devices

		if (SUCCEEDED(hr))
		{
			hr = pDevEnum->CreateClassEnumerator (CLSID_VideoInputDeviceCategory, &pClassEnum, 0);
			if (FAILED(hr))
			{
				printf(TEXT("Couldn't create class enumerator!  hr=0x%x"), hr);
				return "No device";
			}
		}

		if (SUCCEEDED(hr))
		{
			// If there are no enumerators for the requested type, then 
			// CreateClassEnumerator will succeed, but pClassEnum will be NULL.
			if (pClassEnum == NULL)
			{				
					printf(TEXT("You need a video capture device, such as a USB WebCam,\r\n"));					
					return "No device";
			}
		}
		
		// Get the FriendlyName of video capture devices on the device list.
		// Note that if the Next() call succeeds but there are no monikers,
		// it will return S_FALSE (which is not a failure).  Therefore, we
		// check that the return code is S_OK instead of using SUCCEEDED() macro.
		std::wstring str = L"";
		if (SUCCEEDED(hr))
		{
			
			while (pClassEnum->Next(1, &pMoniker, NULL) == S_OK)
			{
				IPropertyBag *pPropBag;
				HRESULT hr = pMoniker->BindToStorage(0, 0, IID_PPV_ARGS(&pPropBag));
				if (FAILED(hr))
				{
					pMoniker->Release();
					continue;  
				} 
				
				VARIANT var;
				VariantInit(&var);

				// Get description or friendly name.
				
				hr = pPropBag->Read(L"FriendlyName", &var, 0);
				
				if (SUCCEEDED(hr))
				{
					//if called from initSession it binds the monikor to the filter object instead of returning a devicelist
					if (DeviceSwitch == true)
					{	
						//Stringcompare to choose the right device
						str.append(var.bstrVal);
						std::string strc = ws2s(str);						
						if (strcmp(Device , strc.c_str()) == 0)
						{
							// Bind Moniker to a filter object
							hr = pMoniker->BindToObject(0,0,IID_IBaseFilter, (void**)&pSrc);							
							if (FAILED(hr))
							{
							   printf(TEXT("Couldn't bind moniker to filter object!  hr=0x%x"), hr);
							}
														
						}
						//VariantClear(&var);
					}
					else{
						// Create string with devicenames 											
						str.append(var.bstrVal);	
						str.append(L"[/]");															
						VariantClear(&var); 
					}
				}
				
				hr = pPropBag->Write(L"FriendlyName", &var);
				
				pPropBag->Release();
				pMoniker->Release();				
				pDevEnum->Release();				
			}									
			pClassEnum->Release();						
		}		
		return ws2s(str);
	}
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

}