/*
 *
 * Copyright (C) 2015 Bernd Böhm
 
 * This toolkit is based on Tim Altmann's 2011 bachelor thesis
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include "webcambase.hpp"
#include <tchar.h>
#include <dshow.h>
namespace WebcamToolkit {
	
class WebcamDShow: public Webcam {
   protected:      
	  bool DeviceSwitch;
	  char* Device;
	  IBaseFilter *pSrc;
	  IVMRWindowlessControl * pWc;
	  ICaptureGraphBuilder2 * g_pCapture;
	  

   public:
      /// initializes components needed for further actions
      WebcamDShow() ;

      /// closes current session and frees reserved memory
      virtual ~WebcamDShow() ;

      /**
       * initalizes a session with given device. If there is already an
       * existing session, this session will be closed and a new session to the
       * given device is opened. The parameters width and height set the size of
       * the recorded picture.
       * returns 1 on success, 0 on error
       * */
      virtual int initSession(int width, int height, char* device);

      /**
       * takes a pictureframe from the webcam and returns its data as RGB.
       * There is no termination symbol but this data is
       * widthImage * heightImage * 3
       * bytes long. The Python wrapper assumes that this memory is allocated
       * using the new[] operator and delete[]s it automatically after creating
       * the Gamera Image.
       * */
      virtual unsigned char* snapshotFromWebcam();

      /// closes the current session and frees memory
      virtual void closeSession();

      /**
       * returns a list of available devices as a string (separated by [/])
       * */
      virtual std::string getDevices();

      
      
      
};
}