/*
 *
 * Copyright (C) 2011 Christian Brandt
 *
 * This toolkit is based on Tim Altmann's 2011 bachelor thesis
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
#ifndef _WEBCAM_V4L_HPP_816392964D7554
#define _WEBCAM_V4L_HPP_816392964D7554
#include "webcambase.hpp"

#define CLEAR(x) memset(&(x), 0, sizeof(x))

namespace WebcamToolkit {

struct buffer;



// -----------------------------------------------------------------------------
class WebcamV4l: public Webcam {
public:
   /// initializes components needed for further actions
   WebcamV4l();

   /// closes current session and frees reserved memory
   ~WebcamV4l();
   
   virtual int initSession(int width, int height, char* device) ;
   virtual unsigned char* snapshotFromWebcam();
   virtual void closeSession();
   virtual std::string getDevices();


protected:
   char* dev_name;
   int fd, isInit;
   unsigned int n_buffers;
   buffer* buffers;

    
   char *getDevicesByName(char* deviceName);
   int stop_capturing(void);
   int uninit_device(void);
   int close_device(void);
   int start_capturing(void);
   int open_device(void);
   int init_device(int width,int height);
   int set_device(char* device);
   void errno_exit (const char* s);
   int xioctl(int fd, int request, void* arg);
   int RGB(int min, int max,int val);
   int read_frame(void);
   void YUY2ToRGB( unsigned char* src, int len, unsigned char* dst );
   int init_mmap(void);
};

} // end WebcamToolkit::

#endif /* _WEBCAM_V4L_HPP_816392964D7554 */

