# -*- mode: python; indent-tabs-mode: nil; tab-width: 3 -*-
# vim: set tabstop=3 shiftwidth=3 expandtab:
#
# Copyright (C) 2011 Christian Brandt
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#

from gamera import toolkit
import platform
import drivers

from gamera.gui import has_gui
import time

Drivers = drivers.driverList
Webcam = drivers.driverInstance

if has_gui.has_gui:
   from gamera.gui import gui
   from gamera.gui import gui_util
   from gamera.args import *
   from gamera.gui import var_name

   class WebcamMenu(toolkit.CustomMenu):
      _items = ["get image"]

      def _Onget_image(self, event):
         available_drivers = drivers.driverList()
         if len(available_drivers) > 0:
            driver = available_drivers[0]
            if len(available_drivers) > 1:
               dialog = Args([Choice("Driver", available_drivers)], "Select driver")

               params = dialog.show()
               driver = available_drivers[params[0]]
         else:
            gui_util.message("No driver found. Please reinstall the webcam toolkit " +\
                       "with drivers for your platform.") 
            return None

         cam = drivers.driverInstance(driver)
         devices = cam.getDevices()         
         if len(devices) == 0:
            gui_util.message("No device found. Please attach a supported video device.")
            return None
         
         dialog = Args([Int("Width", default=640),
                       Int("Height", default=480),
                       Choice("Camera", devices),
                       Int("Wait", default=1),
#                       Int("time", default=100)
                       ],
                       "get Image from camera",
                      )

         params = dialog.show(docstring='''This takes a single frame from the 
webcam. *width* and *height* have to be proportional to the native resolution 
of the camera.

   - *Width* is width of the resulting image.
   - *Height* is height of the resulting image.
   - *Camera* is the device which the image is taken from.
   - *Wait* is the number of images taken every 200 ms before the real image is 
     saved. This can be used for cameras which take some time to focus.

''')


         if params != None and params[2] != None:
            try:
               progress = gui_util.ProgressBox("waiting for calibration ...", 
                                            params[3] + 1) 
               cam.initSession(params[0], params[1], devices[params[2]])
               progress.step()

               for i in range(params[3]):
                  cam.snapshotFromWebcam()            
                  time.sleep(0.2)
   #               time.sleep(params[4]/1000.0)
                  progress.step()

               img = cam.snapshotFromWebcam()
               cam.closeSession()

               name = var_name.get("webcam", gui.main_win.shell.locals)
               if name != None:
                  gui.main_win.shell.locals[name] = img

                  gui.main_win.shell.update()

               progress.kill()
            except Exception, e:
               gui_util.message("%s" % e)

   webcam_menu = WebcamMenu()



