# -*- mode: python; indent-tabs-mode: nil; tab-width: 3 -*-
# vim: set tabstop=3 shiftwidth=3 expandtab:
#
# Copyright (C) 2011 Tim Altmann, Christian Brandt
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#

import _driverwrapper
from gamera.plugin import *

class CreateInstance(PluginFunction):
   args = Args([String("Driver", "default")])
   self_type = None
   return_type=Class("PyObject")

   def __call__(self, driver="default"):
      return _driverwrapper.CreateInstance(driver)

   __call__ = staticmethod(__call__)

class DeleteInstance(PluginFunction):
   self_type = None
   args = Args([Class("PyObject")])

class InitWebcam(PluginFunction):
   self_type = None
   args = Args([Class("PyObject"),Int("Width"), Int("Height"),String("Device")])
   return_type=Int("return")

class GetDevices(PluginFunction):
   self_type = None
   args = Args([Class("PyObject")])
   return_type=String("Devices")

class Snapshot(PluginFunction):
   self_type = None
   args = Args([Class("PyObject")])
   return_type = ImageType([RGB], "Webcam")

class CloseWebcam(PluginFunction):
   args = Args([Class("PyObject")])
   self_type = None

class CppDrivers(PluginFunction):
   self_type = None
   return_type = String("Devices")

class DriverWrapperModule(PluginModule):
   cpp_headers=["driverwrapper.hpp"]
   cpp_namespace=["Gamera"]
   cpp_include_dirs=["drivers"]
   category = "LinuxWebcam"
   functions = [CreateInstance,DeleteInstance,GetDevices,InitWebcam,Snapshot,CloseWebcam,CppDrivers]
   author = "Tim Altmann, Christian Brandt"
   url = "http://gamera.sf.net/"

module = DriverWrapperModule()

CreateInstance= CreateInstance()
DeleteInstance= DeleteInstance()
GetDevices= GetDevices()
InitWebcam= InitWebcam()
Snapshot= Snapshot()
CloseWebcam= CloseWebcam()
CppDrivers = CppDrivers()
