/*
 *
 * Copyright (C) 2011 Tim Altmann, Christian Brandt
 *
 * This toolkit is based on Tim Altmann's 2011 bachelor thesis
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
#ifndef _DRIVERWRAPPER_HPP_C2B94DD732A84C
#define _DRIVERWRAPPER_HPP_C2B94DD732A84C

#include "gamera.hpp"

#include "webcambase.hpp"
namespace WebcamToolkit {
   std::string availableDrivers();
}

using namespace WebcamToolkit;
namespace Gamera {

// -----------------------------------------------------------------------------
template <class T>
void fill_image_from_char(T &image, unsigned char* s) {	
   typedef typename T::value_type value_type;
   typename T::vec_iterator i = image.vec_begin();
   value_type* j = (value_type*)s;
   
   for (; i != image.vec_end(); ++i, ++j) {
      *i = *j;	  
   }   
}



// -----------------------------------------------------------------------------
PyObject* CreateInstance(const std::string& drivername) {
   void* webcam;
   webcam = newWebcam(drivername);
   return PyCObject_FromVoidPtr(webcam,NULL);
}



// -----------------------------------------------------------------------------
void DeleteInstance(PyObject* webcamInstance) {
   Webcam* webcam = (Webcam*)PyCObject_AsVoidPtr(webcamInstance);
   delete webcam; //closes session automatically
}



// -----------------------------------------------------------------------------
int InitWebcam(PyObject* webcamInstance,int width, int height, char* device) {
   Webcam* webcam;
   webcam=(Webcam*)PyCObject_AsVoidPtr(webcamInstance);
   return webcam->initSession(width,height,device);
}



// -----------------------------------------------------------------------------
void CloseWebcam(PyObject* webcamInstance) {
   Webcam* webcam = (Webcam*)PyCObject_AsVoidPtr(webcamInstance);
   webcam->closeSession();
}



// -----------------------------------------------------------------------------
Image* Snapshot(PyObject* webcamInstance) {
   unsigned char* webcamImageData;
   Webcam* webcam = (Webcam*)PyCObject_AsVoidPtr(webcamInstance);   
   webcamImageData= webcam->snapshotFromWebcam();  
   typedef TypeIdImageFactory<RGB, DENSE> factory;
   typedef factory::image_type image_type;	
   image_type* webcamImage = factory::create(Point(0,0),
      Dim(webcam->getImageWidth(), webcam->getImageHeight()));		
	
   if (webcamImageData!=NULL) {	   
      fill_image_from_char(*webcamImage,webcamImageData);
   }
  
   delete [] webcamImageData;
   
   return webcamImage;
}



// -----------------------------------------------------------------------------
std::string GetDevices(PyObject* webcamInstance) {
   Webcam* webcam = (Webcam*)PyCObject_AsVoidPtr(webcamInstance);
   if(webcam == NULL)
      return "No device";
   return webcam->getDevices();
}


std::string CppDrivers() {
   return availableDrivers();
}

} // end Gamera::



#endif /* _DRIVERWRAPPER_HPP_C2B94DD732A84C */

