/*
 *
 * Copyright (C) 2011 Christian Brandt
 *
 * This toolkit is based on Tim Altmann's 2011 bachelor thesis
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

#ifndef _WEBCAM_BASE_HPP_06A6285D10E8D5
#define _WEBCAM_BASE_HPP_06A6285D10E8D5

#include <string>

namespace WebcamToolkit {

class Webcam;
extern Webcam* newWebcam(std::string);



// -----------------------------------------------------------------------------
class Webcam {
   protected:
      int widthImage, heightImage;

   public:
      /// initializes components needed for further actions
      Webcam() {};

      /// closes current session and frees reserved memory
      virtual ~Webcam() {};
 
      /**
       * initalizes a session with given device. If there is already an 
       * existing session, this session will be closed and a new session to the
       * given device is opened. The parameters width and height set the size of
       * the recorded picture.
       * returns 1 on success, 0 on error
       * */
      virtual int initSession(int width, int height, char* device) = 0;
      
      /**
       * takes a pictureframe from the webcam and returns its data as RGB.
       * There is no termination symbol but this data is
       * widthImage * heightImage * 3
       * bytes long. The Python wrapper assumes that this memory is allocated 
       * using the new[] operator and delete[]s it automatically after creating 
       * the Gamera Image.
       * */
      virtual unsigned char* snapshotFromWebcam() = 0;

      /// closes the current session and frees memory
      virtual void closeSession() = 0;

      /// returns a list of available devices separated by [/]
      virtual std::string getDevices() = 0;

      /// returns set image width	  
      int getImageWidth() {		
         return widthImage;
      }

      /// returns set image height	  
      int getImageHeight() {
        return heightImage;
      }
};

} // end WebcamToolkit::

#endif /* _WEBCAM_BASE_HPP_06A6285D10E8D5 */

