#!/usr/bin/env python
import platform
from distutils.core import setup, Extension
import distutils 
from gamera import gamera_setup
import os
import shutil, platform
import sys

# This constant should be the name of the toolkit
TOOLKIT_NAME = "webcam"

# ----------------------------------------------------------------------------
# You should not usually have to edit anything below, but it is
# implemented here and not in the Gamera core so that you can edit it
# if you need to do something more complicated (for example, building
# and linking to a third- party library).
# ----------------------------------------------------------------------------

f = open("VERSION", "r")
TOOLKIT_VERSION = f.read().split("\n")[0]
f.close()

PLUGIN_PATH = 'gamera/toolkits/%s/plugins/' % TOOLKIT_NAME
PACKAGE = 'gamera.toolkits.%s' % TOOLKIT_NAME
PLUGIN_PACKAGE = PACKAGE + ".plugins"
plugins = gamera_setup.get_plugin_filenames(PLUGIN_PATH)
plugin_extensions = gamera_setup.generate_plugins(plugins, PLUGIN_PACKAGE)

plugin_extensions[0].sources.append(os.path.join("drivers", "_selectdriver.cpp"))


def generate_selectdriver():
   print "Generating _selectdriver.cpp"
   drivers_cpp = []
   drivers_python = {}

   for d in os.listdir("drivers"):
      #print d
      if d.startswith("webcam"):
         parameters = {"SETUP_PATH": os.getcwd()}
         #print parameters
         execfile(os.path.join("drivers", d, "driverinfo.py"), parameters)
         #print parameters
         if platform.system() in parameters["platforms"]:
            print d
            if parameters["compile"]:   
               drivers_cpp.append(parameters["classname"])
            if len(parameters["pythonfiles"]) > 0:
               drivers_python[d.lower()] = parameters["pythonfiles"]
            
   #create python
   python = u""
   for d in drivers_python:
      python += "import %s\n" % d

   f = open(os.path.join("drivers","_pythondrivers.py"), "w")
   f.write(python)
   f.close()


   #create cpp
   cpp = u""
   cpp += '''#include <string>
#include <webcambase.hpp>
'''

   for d in drivers_cpp:
      cpp += "#include <%s/%s.hpp>\n" % (d.lower(), d.lower())


   cpp += '''
namespace WebcamToolkit {
Webcam* newWebcam(std::string name) {'''

   for d in drivers_cpp:
      cpp += '''
   if(name == "%s" || name == "default")
      return new %s;
''' % (d, d)

   cpp += '''
   return NULL;
}

std::string availableDrivers() {
   return "'''

   for i,d in enumerate(drivers_cpp):
      if i > 0:
         cpp += "[/]"
      cpp += d

   cpp += '''";
}

}

'''

   f = open(os.path.join("drivers","_selectdriver.cpp"), "w")
   f.write(cpp)
   f.close()


def clear_selectdriver():
   os.remove(os.path.join("drivers","_selectdriver.cpp"))
   os.remove(os.path.join("drivers","_pythondrivers.py"))



def possible_drivers():
   drivers_python = {}
   drivers_cpp = {}
   for d in os.listdir("drivers"):
      if d.startswith("webcam"):
         parameters = {"SETUP_PATH": os.getcwd()}
         #print parameters
         execfile(os.path.join("drivers", d, "driverinfo.py"), parameters)
         if platform.system() in parameters["platforms"]:
            if parameters["compile"]:   
               drivers_cpp[d] = parameters
            if len(parameters["pythonfiles"]) > 0:
               drivers_python[d.lower()] = parameters["pythonfiles"]

   return drivers_cpp, drivers_python

for arg in sys.argv:
   if arg == "build" or arg == "bdist_wininst":
      generate_selectdriver()
      break

drivers_cpp, drivers_python = possible_drivers()

for name, driver in drivers_cpp.items():
   if driver.has_key("syscommands_before") and "build" in sys.argv:
      for cmd in driver["syscommands_before"]:
         print cmd
         if os.system(cmd) != 0:
            raise RuntimeError("Error in syscommands_before of driver " + name)
   
   if driver.has_key("extra_compile_args"):
      plugin_extensions[0].extra_compile_args.extend(driver["extra_compile_args"])
   if driver.has_key("sources"):
      plugin_extensions[0].sources.extend(
         [os.path.join("drivers", name, d) for d in driver["sources"]])

   if driver.has_key("extra_link_args"):
      plugin_extensions[0].extra_link_args.extend(driver["extra_link_args"])
   if driver.has_key("library_dirs"):
      plugin_extensions[0].library_dirs.extend(driver["library_dirs"])
   if driver.has_key("libraries"):
      plugin_extensions[0].libraries.extend(driver["libraries"])

   
   if driver.has_key("copy_on_install") and 'install' in sys.argv:
      for src,dst in driver["copy_on_install"]:
         shutil.copy(os.path.join("drivers", src), dst)

   if driver.has_key("syscommands_after") and 'build' in sys.argv:
      for cmd in driver.syscommands_after:
         print cmd
         if os.system(cmd) != 0:
            raise RuntimeError("Error in syscommands_after of driver " + name)

pythondata = {}
pythonpackages = []
for name, files in drivers_python.items():
   pythonpackages.append("gamera.toolkits.webcam.drivers.%s" % name)
   pythondata["gamera.toolkits.webcam.drivers.%s" % name] = files
   
if "clean" in sys.argv:
   clear_selectdriver()

setup(name=TOOLKIT_NAME, version=TOOLKIT_VERSION,
      ext_modules = plugin_extensions,
      author = "Tim Altmann, Christian Brandt, Bernd Boehm", 
      author_email = "gamera-devel@yahoogroups.com",
      url = "http://gamera.sourceforge.net/",
      package_dir = {"gamera.toolkits.webcam.drivers": "drivers"},
      packages = [PACKAGE, PLUGIN_PACKAGE, "gamera.toolkits.webcam.drivers"] + pythonpackages,
      package_data = pythondata
      )


# This is a standard distutils setup initializer.  If you need to do
# anything more complex here, refer to the Python distutils documentation.



